import { Raycaster, Vector2, Mesh, SphereBufferGeometry, MeshBasicMaterial } from 'three';

class MouseTracker {
    constructor(opts) {
        var _a, _b;
        this.intersects = [];
        this.mesh = opts.mesh;
        this.camera = opts.camera;
        this.tracker = (_a = opts.tracker) !== null && _a !== void 0 ? _a : (() => null);
        this.debug = (_b = opts.debug) !== null && _b !== void 0 ? _b : false;
        this.raycaster = new Raycaster();
        this.pointer = new Vector2();
        if (this.debug) {
            this.generateMouse();
        }
        this.addRaycasterEvent();
    }
    addRaycasterEvent() {
        window.addEventListener('pointermove', (event) => {
            // calculate pointer position in normalized device coordinates
            // (-1 to +1) for both components
            var _a;
            this.pointer.x = (event.clientX / window.innerWidth) * 2 - 1;
            this.pointer.y = -(event.clientY / window.innerHeight) * 2 + 1;
            this.raycaster.setFromCamera(this.pointer, this.camera);
            this.intersects.splice(0, this.intersects.length, ...this.raycaster.intersectObjects([this.mesh]));
            if (this.intersects[0]) {
                this.tracker(this.intersects[0].point);
                if (this.debug) {
                    (_a = this.object) === null || _a === void 0 ? void 0 : _a.position.copy(this.intersects[0].point);
                }
            }
        });
    }
    generateMouse() {
        this.object = new Mesh(new SphereBufferGeometry(0.1, 10, 10), new MeshBasicMaterial({ color: 0xff0000, wireframe: true }));
    }
}

export { MouseTracker as default };
